#!/usr/local/bin/wish

##############################################################################
# DBFastER - DBFastER: DataBase Fasta HeadER modification                    #
#                                                                            #
# DBFastER has been created to modify fasta header provided by NCBI (for ex) #
# in order to improve the results for creating BLAST database for ex.        #
#                                                                            #
# 2006 Jean Muller (jean.muller@igbmc.u-strasbg.fr)                          # 
#                                                                            #
##############################################################################

############################################
# Recherche des fichiers sources           #
# Positionnement des variables importantes #
############################################
proc ProgSourceDir {} {

    #Permet de determiner la racine des fichiers sources .tcl

    global ProgSourceDir
    if {[info exists ProgSourceDir] && [file exists $ProgSourceDir]} {return $ProgSourceDir}

    set Script [info script]

    if {$Script==""} {
	tk_messageBox -message "I cannot find the script directory"
	set ProgSourceDir "."
    } else {
	if {[catch {
	    while {[file exists $Script] && [file type $Script]=="link"} {
		set RepScript [file dirname $Script]
		if {[catch {set Script [file readlink $Script]} Message]} { break }
		if {[file pathtype $Script]=="relative"} {
		    while {[regexp {^\.\./} $Script]} {
			regsub {^\.\./} $Script "" Script
			set RepScript [file dirname $RepScript]
		    }
		    set Script "$RepScript/$Script"
		}
	    }
	} Message]} {
	    #tk_messageBox -message "Plante avec message \n$Message"
	}
	set ProgSourceDir [file dirname $Script]
    }

    global InfoScript ; set InfoScript $Script
    return $ProgSourceDir
}

proc DBFastER_Version {} {

    #Version de DBFastER

    global DBFastER_Version

    if {[info exists DBFastER_Version]} {return $DBFastER_Version}	
    set     DBFastER_Version "1.0"

    return $DBFastER_Version
}

proc HOMEDIR {} {

    #Home directory du user en cours

    global HOMEDIR
    global env

    if {[info exists HOMEDIR] && [file exists $HOMEDIR]} {return $HOMEDIR}	
    set     HOMEDIR ""
    if {[info exists env(HOME)]} {
	set HOMEDIR [file join [set env(HOME)]]
    }
    if {   $HOMEDIR == ""} {
	set HOMEDIR [file join [ProgSourceDir]]
    }
    return $HOMEDIR
}

proc DBFastER_Panel {} {

    #Manager de modification de header de fasta
    #DataBase Fasta Header Modification
    
    #ATTENTION Ajouter une fonction 

    #On gere la premiere fenetre en . si il y deja autre chose d utiliser dans .
    #ATTENTION on doit modifier . en "" pour pouvoir conserver la hierarchie des fenetres
    if {[winfo children .] != ""} {
	set       w ".dbfastermanagement"
	if {[winfo exists $w]} {return ""}
	toplevel $w
    } else {
	set w "."
    }
    #if {[winfo exists $w]} {wm deiconify $w;raise $w;return ""} else {toplevel $w}

    wm geometry  $w +300+100
    wm title     $w "DBFastER [DBFastER_Version]"
    wm resizable $w false false
    #wm resizable $w true true
    wm protocol  $w WM_DELETE_WINDOW "destroy $w"
    if {$w == "."} {set w ""}

    #Frame Generale contenant les autres frame et le bouton exit
    set     FirstFrame  "$w.framehome"
    set     LabelTitle  "$FirstFrame.ltitle"
    set     LabelTitle2 "$FirstFrame.ltitle2"
    set     ButtonExit  "$FirstFrame.bdismiss"

    frame  $FirstFrame  -borderwidth "3"     -relief "groove"
    label  $LabelTitle  -background "orange" -relief "ridge" -text "Fasta Header Management" -padx "2" -anchor "center"
    if {$w ==  ""} {set w "."}
    button $ButtonExit  -background "red"    -relief "ridge" -text "Exit"                    -padx "2" -command "eval \[wm protocol $w WM_DELETE_WINDOW\]"
    if {$w == "."} {set w ""}

    #1 Input Fasta File Panel
    global  EBrowseFasta
    set     FBrowseFasta "$FirstFrame.fbrowsefasta"
    set     LBrowseFasta "$FBrowseFasta.lbrowsefasta"
    set     EBrowseFasta "$FBrowseFasta.ebrowsefasta"
    set     BBrowseFasta "$FBrowseFasta.bbrowsefasta"

    frame  $FBrowseFasta -borderwidth "3"        -relief "groove"
    label  $LBrowseFasta -background "#808FFF"   -relief "groove" -text "Fasta file:" -width "10" -anchor "w"
    entry  $EBrowseFasta -textvariable BrowseFasta                                   
    button $BBrowseFasta -background "yellow"    -relief "groove" -text "Browse"      -padx   "2" -command {
	set FileTypes {
	    {{Fasta Files} {.tfa}}
	    {{Fasta Files} {.fa}}
	    {{Fasta Files} {.fasta}}
	    {{Text Files} {.txt}}
	    {{All Files}  *     }
	}
	set BrowsedFasta [tk_getOpenFile -multiple "n" -title "Choose 1 Fasta file." -initialdir "[HOMEDIR]" -filetypes $FileTypes]
	$EBrowseFasta delete 0 end
	$EBrowseFasta insert   end $BrowsedFasta

	if {$BrowseFasta == ""} {
	    AjouteAuLogFastER "No Fasta file selected."
	} else {
	    set  FileFormat [FastaFileFormat $BrowseFasta]
	    AjouteAuLogFastER "Selected file is $FileFormat."
	}
    }

    #2 Output File Panel
    global  ESaveFasta
    set     FSaveFasta "$FirstFrame.fsavefasta"
    set     LSaveFasta "$FSaveFasta.lsavefasta"
    set     ESaveFasta "$FSaveFasta.esavefasta"
    set     BSaveFasta "$FSaveFasta.bsavefasta"

    frame  $FSaveFasta -borderwidth "3"      -relief "groove"
    label  $LSaveFasta -background "#808FFF" -relief "groove" -text "Output file:" -width "10" -anchor "w"
    entry  $ESaveFasta -textvariable SaveFasta	      
    button $BSaveFasta -background "yellow"  -relief "groove" -text "Browse"       -padx   "2" -command {
	set SavedFasta [tk_getSaveFile -title "Choose a file name to save" -initialdir "[HOMEDIR]"]
	if {[file exists $SavedFasta]} {
	    file delete -force $SavedFasta
	}
	if {$SavedFasta != ""} {
	    $ESaveFasta delete 0 end
	    $ESaveFasta insert   end $SavedFasta
	    
	    AjouteAuLogFastER "Output file is $SaveFasta."
	} else {
	    AjouteAuLogFastER "No Output file selected."
	}
    }

    #3 Prefix Selection
    set    FPrefixFasta  "$FirstFrame.fprefixfasta"
    set    LPrefixFasta  "$FPrefixFasta.lprefixfasta"
    set    LPrefixFasta2 "$FPrefixFasta.lprefixfasta2"
    set    EPrefixFasta  "$FPrefixFasta.eprefixfasta"

    frame $FPrefixFasta  -borderwidth "3"      -relief "groove"
    label $LPrefixFasta  -background "#808FFF" -relief "groove" -text "Prefix" -width "10" -anchor "w"
    entry $EPrefixFasta  -textvariable Prefix
    label $LPrefixFasta2 -background "#808FFF" -relief "groove"                -width "10"

    #4 Log Panel
    global LBLogFastER
    set        FLogFastER   "$FirstFrame.flogfaster"
    set        LBLogFastER  "$FLogFastER.lblogfaster"
    set        SBVLogFastER "$FLogFastER.yscroll"
    set        FLogFastER2  "$FirstFrame.flogfaster2"
    set        BLogFastER   "$FLogFastER2.blogfaster"
    set        LLogFastER1  "$FLogFastER2.llogfaster1"
    set        LLogFastER2  "$FLogFastER2.llogfaster2"
    set        BHelp        "$FLogFastER2.bhelp"
    set        BRun         "$FLogFastER2.brun"

    frame     $FLogFastER   -background "#808080"  -relief  "groove"
    scrollbar $SBVLogFastER -orient     "vertical" -command "$LBLogFastER yview"
    text      $LBLogFastER  -state      "disabled" -wrap    "word"   -width "30"       -height "14" -yscrollcommand "$SBVLogFastER set"
    frame     $FLogFastER2  -background "#808080"  -relief  "groove"
    button    $BLogFastER   -background "yellow"   -relief  "groove" -text "Reset Log" -padx "3"    -command "EffaceLeLogFastER $LBLogFastER"
    label     $LLogFastER1  -background "#808FFF"  -relief  "groove" 
    label     $LLogFastER2  -background "#808FFF"  -relief  "groove" 
    button    $BHelp        -background "orange"   -relief  "groove" -text "Help"      -padx "3"    -command {
	AjouteAuLogFastER ""
	AjouteAuLogFastER "------- Help -------"
	AjouteAuLogFastER "DBFastER allows you to manipulate Fasta formatted files, especially for NCBI Database files (e.g. RefSeq, Unigene...)."
	AjouteAuLogFastER ""
	AjouteAuLogFastER "1- Database fasta-formatted file."
	AjouteAuLogFastER "2- Output file."
	AjouteAuLogFastER "3- (Option) Database prefix."
	AjouteAuLogFastER "4- Run to modify your database header by keeping only the Ids after the \">\"."

	AjouteAuLogFastER "Remark: Prefix to database Ids (e.g. for UniGene \"UG:\" or RefSeq \"RS:\")"
	AjouteAuLogFastER "-------------------------------"
    }
    button    $BRun         -background "green"    -relief  "groove" -text "Run"       -padx "3"    -command {
	if {! [info exists BrowsedFasta] || ([info exists BrowsedFasta] && ($BrowsedFasta == "" || ! [file exists $BrowsedFasta]) )} {
	    AjouteAuLogFastER "No fasta file selected."
	    return ""
	}
	if {! [info exists SavedFasta] || ([info exists SavedFasta] && $SavedFasta == "")} {
	    AjouteAuLogFastER "No output file selected."
	    return ""
	} elseif {[info exists SavedFasta] && [file exists $SavedFasta]} {
	    file delete -force $SavedFasta
	}

	AjouteAuLogFastER "Applying changes..."
	AjouteAuLogFastER "...Changing headers..."
	
	FaireAttendre .waitload "Waiting" "Please wait ...\nChanging headers ..."
	ModifyFastaHeader_DBFastER $BrowsedFasta $SavedFasta $Prefix
	FaireLaSuite  .waitload
	
	AjouteAuLogFastER "...Changes applied."
    }

    pack $FirstFrame                -fill both 
    pack $LabelTitle    -side top   -fill both

    pack $FBrowseFasta  -side top   -fill both -expand true
    pack $LBrowseFasta  -side left  -fill both
    pack $EBrowseFasta  -side left  -fill both -expand true
    pack $BBrowseFasta  -side left  -fill both 

    pack $FSaveFasta    -side top   -fill both -expand true
    pack $LSaveFasta    -side left  -fill both
    pack $ESaveFasta    -side left  -fill both -expand true
    pack $BSaveFasta    -side left  -fill both 
    
    pack $FPrefixFasta  -side top   -fill both -expand true
    pack $LPrefixFasta  -side left  -fill both 
    pack $EPrefixFasta  -side left  -fill both -expand true
    pack $LPrefixFasta2 -side left  -fill both 

    pack $FLogFastER    -side top   -fill both -expand true
    pack $SBVLogFastER  -side right -fill y
    pack $LBLogFastER               -fill both -expand true
    pack $FLogFastER2   -side top   -fill x   
    pack $LLogFastER1   -side left  -fill both -expand true
    pack $BLogFastER    -side left  -fill both
    pack $BHelp         -side left  -fill both 
    pack $BRun          -side left  -fill both 
    pack $LLogFastER2   -side left  -fill both -expand true

    pack $ButtonExit    -side top   -fill x

    AjouteAuLogFastER "---- Welcome to DBFastER [DBFastER_Version] ----"
    AjouteAuLogFastER ""
    AjouteAuLogFastER "DataBase Fasta HeadER modification, a program to modify fasta header."
    AjouteAuLogFastER "Please use Help button for more informations."
    AjouteAuLogFastER "Please feel free to contact us for any suggestions or bug reports."
    AjouteAuLogFastER "email: jean.muller@igbmc.u-strasbg.fr"
    AjouteAuLogFastER ""
     
    if {$w == ""} {set w "."}
    return ""
}

#############################
#  GUI proc additionnelles  #
#############################
proc FaireAttendre {w ti tx} {

    Wup "Permet de faire attendre le user pendant que le prog travaille"
    Wup "Fait apparaitre une boite de dialogue pour dire ce qui se passe"

    global oldGrab
    global oldFocus
    global grabStatus
 
    catch {destroy $w}
    
    toplevel $w
    wm title $w $ti

    if {[winfo viewable [winfo toplevel [winfo parent $w]]] } {
        wm transient $w [winfo toplevel [winfo parent $w]]
    }

    label $w.label -background red -text $tx -justify left
    pack  $w.label -padx 10 -pady 10
    
    wm withdraw $w
    set x [expr {[winfo screenwidth  $w]/2 - [winfo reqwidth  $w]/2  - [winfo vrootx [winfo parent $w]]}]
    set y [expr {[winfo screenheight $w]/2 - [winfo reqheight $w]/2  - [winfo vrooty [winfo parent $w]]}]

    wm geom $w +$x+$y
    wm deiconify $w

    update idletasks

    set oldFocus [focus]
    set oldGrab  [grab current $w]
    if {[string compare $oldGrab ""]} {
        set grabStatus [grab status $oldGrab]
    }

    grab  $w
    focus $w

    #update idletasks
    update
    #tkwait visibility $w.label
    
    return ""
}

proc FaireLaSuite {w} {

    Wup "Permet de faire attendre le user pendant que le prog travaille"
    Wup "Fait diapparaitre la boite de dialogue et remet le focus sur la precedente fenetre"

    global oldGrab
    global oldFocus
    global grabStatus

    catch {focus $oldFocus}
    catch {destroy $w}

    if {[string compare $oldGrab ""]} {
      if {[string compare $grabStatus "global"]} {
	  catch {grab $oldGrab}
      } else {
          catch {grab -global $oldGrab}
      }
  }

  update idletasks
  
  return ""
}

proc AjouteAuLogFastER {Log} {
  
    Wup "Permet d ecrire dans le log du panel general (GUI) ou par defaut d ecrire sur la sortie terminal"

    global LBLogFastER

    if {! [info exists LBLogFastER]} {puts "$Log";return ""}

    $LBLogFastER configure -state normal
    $LBLogFastER insert    "end"  "$Log\n"
    $LBLogFastER configure -state disabled
    #Pour se positionner a la fin du texte
    $LBLogFastER see end
    return ""
}

proc EffaceLeLogFastER {LBLog} {

    Wup "Permet d effacer le log du panel general"

    global $LBLog
    
    if {$LBLog==""} {return ""}
    
    $LBLog configure -state normal
    $LBLog delete    0.0    end
    $LBLog configure -state disabled
    return ""
}

proc FastaFileFormat {Fichier {Quoi ""}} {

    Wup "Permet de tester et de decider pour un fichier fasta multiple"
    Wup "Retourne vide si rien n est fait et le nom des fichiers si des fichiers sont crees"

    global FastaType

    if {$Quoi == "Reset"} {
	array unset FastaType "$Fichier*"
	return ""
    }
    if {[info exists FastaType($Fichier)]} {return [set FastaType($Fichier)]}

    set Type [CheckTFAFile $Fichier]
    set FastaType($Fichier) $Type

    return [FastaFileFormat $Fichier]
}

proc CheckTFAFile {Fichier} {

    Wup "Permet de tester et de decider pour un fichier fasta multiple"
    Wup "Retourne vide si rien n est fait et le nom des fichiers si des fichiers sont crees"

    if {$Fichier == "" || ! [file exists $Fichier]} {return ""}

    set LesNewFiles ""
    set NbChevrons [EstCeUnTFAOuUnTFAS $Fichier]
    if {$NbChevrons=="0"} {
	set Type "Unknown"
    } elseif {$NbChevrons=="1"} {
	set Type "Fasta"
    } elseif {$NbChevrons>1} {
	set Type "Multiple Fasta"
    } 
    
    return "$Type"
}

proc EstCeUnTFAOuUnTFAS {{Fichier ""}} {

    Wup "Test si le fichier est un tfa multiple ou pas"
    Wup "Retourne le nombre de chevrons par fichier"

    set nbChevrons 0
    set nbLigne    0
    set F [open "$Fichier"]
    while {[gets $F Ligne]>=0} {
	if {! [regexp -nocase {[a-z0-9]+} $Ligne]} {continue}
	incr nbLigne
	if {  [string range $Ligne 0 0] == ">"} {
	    incr nbChevrons
	}
	#Attention on doit trouver au moins un chevron dans les 3 premieres lignes
	if {$nbChevrons=="0" && $nbLigne > 3} {break}
	#Si on trouve plus de 2 chevrons on peut s arreter
	if {$nbChevrons>2} {break}
    }
    close $F

    return "$nbChevrons"
}

proc ModifyFastaHeader_DBFastER {File FileOut {Prefixe ""}} {

    Wup "Modify the fasta header by replacing the header by keeping only the accession number and adding refix (option)"
    Wup "Formate le fichier Fasta de RefSeq (NCBI) pour avoir uniquement le NM_ au debut"
    Wup ">gi|4758487|ref|NM_004128.1| Homo sapiens general transcrip"
    Wup ">NM_004128 Homo sapiens general transcrip"
    
    set F  [open "$File"]
    while {[gets $F Ligne]>=0} {
	if {! [regexp -nocase {[a-z0-9]+} $Ligne]} {continue}
	#Unigene
	#>gnl|UG|Hs#S19185843 Homo sapiens N-acetyltransferase 2 (arylamine N-acetyltransferase), mRNA (cDNA clone MGC:71963 IMAGE:4722596), complete cds /cds=p(105,977) /gb=BC067218 /gi=45501306 /ug=Hs.2 /len=1344
	#GGGGACTTCCCTTGCAGACTTTGGAAGGGAGAGCACTTTATTACAGACCTTGGAAGCAAG
	#RefSeq
	#>gi|10863904|ref|NM_004239.1| Homo sapiens thyroid hormone receptor interactor 11 (TRIP11), mRNA
	#CGAGCGAGTGTCATGGCGGCCGGCGTCGAGTTGGCAGGAGTAACCCACGGAACTGAGGAAAGTCATTAGAGCTGAGAAAG

	if {[string first ">" $Ligne]=="0"} {
	    set Entete  ""
	    set LEntete ""
	    set Debut   ""
	    set Fin     ""

	    set LigneSplitee [split $Ligne " "]
	    set Entete       [lindex $LigneSplitee 0]
	    regsub -all {\|} $Entete " " Entete
	    set Entete       [string trim $Entete]
	    set LEntete      [split $Entete " "]
	    
	    set Debut        [lindex $LEntete end]
	    set Fin          [join [lrange $LigneSplitee 1 end] " "]

	    if {[regexp {\.[0-9]+$} $Debut]} {
		regsub  {\.[0-9]+$} $Debut "" Debut
	    }
	    set Ligne ">${Prefixe}${Debut} $Fin"
	}
	lappend LesLignes $Ligne
    }
    close $F
    
    AppendAuFichier $FileOut "[join $LesLignes "\n"]"

    return ""
}

proc AppendAuFichier {Fichier Ligne} {
    set    Canal [open $Fichier "a"]
    puts  $Canal $Ligne
    close $Canal
    return $Fichier
}

proc Wup Texte {
 
}

#############################################
################# DBFastER ##################
#############################################
if {$argv!={}} {
    DBFastER_Panel
} else {
    DBFastER_Panel
}
#################### FIN ####################

